#ifndef _SENSOR_H
#define _SENSOR_H

/* Constants */
#define GRAVITY (9.81F) // m/s^2

/* =================================== */
/* Sensor Information */
/* =================================== */

/* Button config */
#define BOUNCE_MS           25
#define SHORT_PRESS         350  // ms
#define LONG_PRESS          1000 // ms

/* Accelerometer config */
#define G_SCALE                     3.0F 
#define ACCELEROMETER_ADC_RANGE     675.0F // 3.3/5.0 * 1023

/* Button Assign */
typedef enum {
    BUTTON_PINKY, 
    BUTTON_RING, 
    BUTTON_MIDDLE, 
    BUTTON_INDEX
};
/* #define PINKY               'p' */
/* #define RING                'r' */
/* #define MIDDLE              'm' */
/* #define INDEX               'i' */

/* Sensor types */
typedef enum {
    SENSOR_TYPE_ACCELEROMETER         = (1),
    SENSOR_TYPE_BUTTON                = (2),
    SENSOR_TYPE_MUSCLE_ACTIVITY       = (3),
} sensors_type_t;

// Taken from adafruit/Adafruit_Sensor on git
/** struct sensors_vec_s is used to return a vector in a common format. */
typedef struct {
    union {
        float v[3];
        struct {
            float x;
            float y;
            float z;
        };
        /* Orientation sensors */
        struct {
            float roll;    /**< Rotation around the longitudinal axis (the plane body, 'X axis'). 
                             Roll is positive and increasing when moving downward. -90<=roll<=90 */
            float pitch;   /**< Rotation around the lateral axis (the wing span, 'Y axis'). 
                             Pitch is positive and increasing when moving upwards. -180<=pitch<=180) */
            float heading; /**< Angle between the longitudinal axis (the plane body) and magnetic north, 
                             measured clockwise when viewing from the top of the device. 0-359 */
        };
    };
    /* int8_t status; */
    /* uint8_t reserved[3]; */
} sensors_vec_t;

/** struct ivec_t is used to return a vector of ints in a common format. */
typedef struct {
    union {
        int v[3];
        struct  {
            int x;
            int y;
            int z;
        };
    };
} ivec_t;

// Button sensor struct
typedef struct {
    union {
        uint8_t data[4];
        struct {
            uint8_t pinky;
            uint8_t ring;
            uint8_t middle;
            uint8_t index;
        };
    };
    /* int8_t status; */
    /* uint8_t reserved[3]; */
} sensors_u08_t;

// Packed attribute specifies each member of the structure or union is placed 
// to minimize the memory required.
// Size: 14 bytes probably padded to 16bytes
typedef struct {
    uint8_t type;         // What type of sensor, accelerometer, button, header etc.
    uint8_t id;               // Unique identifier for this sensor
    union {
        float data[3];           
        sensors_vec_t acceleration; // m/s^2
        sensors_vec_t position;     // m
    };
} __attribute__((__packed__)) f_sensor_packet_t; 

// Packed attribute specifies each member of the structure or union is placed 
// to minimize the memory required.
// Size: 14 bytes probably padded to 16bytes
typedef struct {
    uint16_t type;         // What type of sensor, accelerometer, button, header etc.
    uint16_t id;               // Unique identifier for this sensor
    union {
        int data[3];           
        ivec_t acceleration; // DAC
    };
} __attribute__((__packed__)) i_sensor_packet_t; 

// Size: 6 bytes
typedef struct {
    uint8_t type;         // What type of sensor, accelerometer, button etc. Using type as the header
    uint8_t id;               // Unique identifier for this sensor
    union {
        uint8_t data[4];           
        sensors_u08_t button;   // which finger is in contact with the thumb
    };
} __attribute__((__packed__)) u08_sensor_packet_t; 

/* // sensor_t is used to provide a single sensor event in a common format */
/* typedef struct { */
/*     uint8_t type;         // What type of sensor, accelerometer, button etc. */
/*     u08 id;               // Unique identifier for this sensor */
/*     union { */
/*         float data[4]; */           
/*         sensors_vec_t acceleration; // m/s^2 */
/*         uint8_t button;       // which finger is active */
/*     } */
/* } sensor_t; */ 


/* typedef struct sensor sensor_t; */

#endif
