import numpy as np
import math as m
import serial
import struct
from IPython.core.debugger import set_trace as trc

def splitList(A):
    half = len(A)//2
    return A[:half], A[half:]

class SerialCommunication:
    ''' Serial Communication class to read and write over serial buffer '''
    def __init__(self, port = 'COM8', baudrate = 115200):
        self.ser = serial.Serial(port = 'COM8', baudrate = 115200);
        self.start_flag = 0x12FF;
        self.end_flag = 0x12EE;
        self.endian = '@'
        self.format = '8H'

    def GetChecksum(self, pkt_in):
        arr = np.array(pkt_in)
        b_arr = arr.tobytes()
        n_arr = np.frombuffer(b_arr,dtype = np.uint8)
        checksum = sum(n_arr) & 0xff
        return checksum

    def ParsePacket(self,data):
        hdr = data[0]
        pkt = data[1:6]
        rdata = data[3:6]
        chksum_in = data[6]
        ftr = data[7]
        in_data = tuple([hdr, rdata, chksum_in, ftr])
        return in_data, pkt

# Go through the buffer array and find the header and end flags
    def ParseMultiData(self, ser_data):
        sflag = self.start_flag
        eflag = self.end_flag
        init_flag = True
        for i,val in enumerate(ser_data):
            if val == sflag:
                tmp_arr = np.array((val))
            else:
                tmp_arr = np.append(tmp_arr, val)

            if val == eflag:
                in_data, pkt = self.ParsePacket(tmp_arr)
                chksum_in = in_data[2]
                chksum = self.GetChecksum(pkt)
                if chksum_in != chksum:
                    print('Error chksum not equal!')
                    continue
                if init_flag:
                    val_arr = np.array((tmp_arr))
                    init_flag = False
                else:
                    val_arr = np.row_stack((val_arr,tmp_arr))

        return val_arr

    def Receiving(self):
        ser1 = self.ser
        sflag = self.start_flag
        eflag = self.end_flag
        in_str = ser1.read(ser1.inWaiting())
        while ser1.inWaiting() > 0:
            in_str += ser1.read(ser1.inWaiting())
            print 'waiting'

        in_len = len(in_str)
        base_fmt = self.endian + self.format
        sz = struct.calcsize(base_fmt)
        n = in_len/sz
        fmt = self.endian + self.format*n

        # print(ser.inWaiting())
        data_in = struct.unpack(fmt,in_str)
        # Take the latest update only
        if in_len > sz:
            buff_str = self.ParseMultiData(data_in)
        else:
            buff_str = data_in

        return buff_str

    def ReadBuff(self, buff_in):
        buff_len = len(buff_in)
        buff_val = [0]*(buff_len/2)
        j = 0
        for i in range(buff_len):
            if i is 0 or (i%2) is 0:
                lobyte = buff_in[i]
                hibyte = buff_in[i+1] << 8
                buff_val[j] = lobyte + hibyte
                j += 1
        return buff_val

