#include "Serial_Communication.h"

uint8_t calculateChecksum(unsigned char * pktbuff, size_t pkt_size) {
    uint8_t checksum = 0;
    for (int i = 0; i < pkt_size; i++) {
        checksum += pktbuff[i];
    }
    checksum &= 0xFF;
    return checksum;
}

Serial_Communication::Serial_Communication() {
    serial_in = NULL;
    update_request = false;
    checksum = 1;
    header = 0x12FF;
    end_byte = 0x12EE;
    ptr = send_pkt;
    message_size = 0;
}

Serial_Communication::~Serial_Communication() {}

void Serial_Communication::Serial_Setup() {
    Serial.begin(BAUDRATE);
    Serial.flush();
}

void Serial_Communication::Read_Update_Request() {
    while (Serial.available() > 0) {
        serial_in = (char)Serial.read();
        if (serial_in == PING) {
            update_request = true;
            break;
        } else if (serial_in != PING) {
            update_request = false;
        }
    }
}

/*
void Serial_Communication::Send_Sensor_Info(sensor_t _sense_data) {
    char out_buffer[40];
    //char float_buff0[10]; // 3 dec places
    //char float_buff1[10]; // 3 dec places
    //char float_buff2[10]; // 3 dec places
    //if (update_request) {
    if (true) {
        snprintf(out_buffer,sizeof(out_buffer),"%d,%d,%d,%d,%d,%c\t",
                _sense_data.sens_type,
                _sense_data.isens_info,
                _sense_data.fsens_info[0],
                _sense_data.fsens_info[1],
                _sense_data.fsens_info[2],
                _sense_data.csens_info);
        Serial.println(out_buffer);
        Serial.flush();
        //int type = _sense_data.sens_type;
        //if (type == BUTTON_TYPE) {
        //    char finger = _sense_data.csens_info;
        //    snprintf(out_buffer,5,"%d,%c\t",type,finger);
        //}
        //Serial.println(out_buffer);
    }
}
*/

int Serial_Communication::Send_u08_Sensor_Packet(u08_sensor_packet_t packet_out) {
    int bytesSent = 0;
    int pkt_size = sizeof(packet_out);
    const char send_pkt[pkt_size + sizeof(checksum) + 1];       // uchar is 1 byte long
    memcpy(&send_pkt, &packet_out, pkt_size);
    memcpy(&send_pkt + pkt_size, &checksum, sizeof(checksum));
    /* messagePack(send_pkt, &packet_out, sizeof(packet_out), checksum); */
        /* (byte*)&packet_out, sizeof(packet_out); */
    
    if (Serial.availableForWrite()) {
        bytesSent = (Serial.write(send_pkt));
        Serial.flush();
    }
    return bytesSent;
}

int Serial_Communication::Send_f_Sensor_Packet(f_sensor_packet_t packet_out) {
    int bytesSent = 0;
    int pkt_size = sizeof(packet_out);
    int len = pkt_size + sizeof(checksum) + sizeof(header) + 
        sizeof(end_byte);
    const unsigned char send_pkt[len];
    unsigned char * ptr;
    ptr = send_pkt;
    /* const char send_pkt ; */
    /* send_pkt = (char *) malloc(len); */
    /* if (send_pkt == NULL) { */
    /*     Serial.println("failed memory allocation"); */
    /* } */
    
    /* Serial.print("Before memcpy: "); */
    /* Serial.print(atoi(*ptr)); */

    memcpy(send_pkt, &header, sizeof(header));
    /* Serial.print("\tAfter header: "); Serial.print((*ptr)); */
    /* Serial.print("::"); */
    /* Serial.print(sizeof(header)); */

    ptr = ptr + sizeof(header);

    memcpy(send_pkt + sizeof(header) , &packet_out, pkt_size);
    checksum = calculateChecksum(ptr, pkt_size);

    /* Serial.print("\tAfter memcpy: "); */
    /* Serial.print((*ptr)); Serial.print("::"); */ 
    /* Serial.print(sizeof(packet_out)); */

    ptr = ptr + sizeof(packet_out);

    /* for (int i = 1; i < pkt_size+1; i++) { */
    /*     checksum += send_pkt[i]; */
    /* } */
    /* checksum &= 0xFF; */

    memcpy(send_pkt + sizeof(header) + pkt_size, 
            &checksum, sizeof(checksum));

    /* Serial.print("\tAfter cksum: "); Serial.print(*ptr); Serial.print("::"); */
    /* Serial.print(sizeof(checksum)); Serial.print("::"); Serial.print(checksum); */

    ptr = ptr + sizeof(checksum);

    memcpy(send_pkt + sizeof(header) + pkt_size + sizeof(checksum), 
            &end_byte, sizeof(end_byte));

    /* ptr = ptr + sizeof(end_byte); */
    /* ++ptr; */

    /* Serial.print("\tAfter headers: ");Serial.print(*ptr); Serial.print("::"); */
    /* Serial.println(sizeof(end_byte)); */

    if (Serial.availableForWrite()) {
        bytesSent = Serial.write(send_pkt,len);
        Serial.flush();
    }
    free(send_pkt);
    return bytesSent;
}

int Serial_Communication::MessagePack(i_sensor_packet_t packet_out) {
    int pkt_size = sizeof(packet_out);
    int len = pkt_size + sizeof(checksum) + sizeof(header) + 
        sizeof(end_byte);

    memcpy(ptr, &header, sizeof(header));

    ptr += sizeof(header);

    memcpy(ptr , &packet_out, pkt_size);

    checksum = calculateChecksum(ptr, pkt_size);

    ptr += pkt_size;

    memcpy(ptr, &checksum, sizeof(checksum));

    ptr += sizeof(checksum);

    memcpy(ptr, &end_byte, sizeof(end_byte));

    ptr += sizeof(end_byte);

    message_size += len;

    /*
    Serial.print(message_size);
    Serial.print('\t');
    for (int i = len; i > 0 ; i--) {
        Serial.print(i); Serial.print(": "); 
        //Serial.print(*(ptr-i));
        Serial.print(send_pkt[message_size-len + i-1]);
        Serial.print('\t');
    }
    Serial.println();
    */

    if (message_size > DEFAULT_BUFFER_LEN - 100) {
        Reset_Buffer();
    }

    return len;
}

void Serial_Communication::Reset_Buffer() {
    ptr -= message_size;
    message_size = 0;
}

int Serial_Communication::Send_Packet() {
    int bytesSent = 0;
    if (Serial.availableForWrite()) {
        bytesSent = Serial.write(send_pkt, message_size);
        Serial.flush();
    }
    return bytesSent;
}

int Serial_Communication::Send_i_Sensor_Packet(i_sensor_packet_t packet_out) {
    int bytesSent = 0;
    int pkt_size = sizeof(packet_out);
    int len = pkt_size + sizeof(checksum) + sizeof(header) + 
        sizeof(end_byte);
    const unsigned char send_pkt[len];
    unsigned char * ptr;

    ptr = send_pkt;
    memcpy(send_pkt, &header, sizeof(header));

    ptr = ptr + sizeof(header);

    memcpy(send_pkt + sizeof(header) , &packet_out, pkt_size);
    checksum = calculateChecksum(ptr, pkt_size);

    ptr = ptr + sizeof(packet_out);

    memcpy(send_pkt + sizeof(header) + pkt_size, 
            &checksum, sizeof(checksum));

    ptr = ptr + sizeof(checksum);

    memcpy(send_pkt + sizeof(header) + pkt_size + sizeof(checksum), 
            &end_byte, sizeof(end_byte));

    if (Serial.availableForWrite()) {
        bytesSent = Serial.write(send_pkt,len);
        Serial.flush();
    }
    free(send_pkt);
    return bytesSent;
}

bool Serial_Communication::Get_Update_Request() {
    return update_request;
}

void Serial_Communication::Set_Update_Request(bool _update_request) {
    update_request = _update_request;
}
