import numpy as np
import math as m
import serial
import time
import struct
import csv

from SerialCommunication import SerialCommunication
from scipy.signal import butter, lfilter, freqs

from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from IPython.core.debugger import set_trace as trc

ENDIAN = '@'
FORMAT = '8H'

def f_map(x, in_min, in_max, out_min, out_max):
  map_vals = (x - in_min) * (out_max - out_min) \
          / (in_max - in_min) + out_min;
  return map_vals

if __name__ == "__main__":
    plt.close('all')
    SC = SerialCommunication()
    ser = SC.ser
    i = 0
    buff = ''

    gravity = 9.81
    g_scale = 3.0

    acc_ax = ['x','y','z']
    adir = 2
    offset = np.zeros((3,3))
    real = np.array(([9.81,0,0], \
                     [0,9.81,0],\
                     [0,0,9.81]))

    bias = np.zeros((3,3))

    for adir in range(0,3):
        raw_input("Calibrate {0} direction\n Setup such that {0} is in-line"
                  "with gravity".format(acc_ax[adir], acc_ax[adir]))
        fig, ax = plt.subplots(3,1, num = adir)
        # ax = fig.add_subplot(111, projection='3d')
        # ax.axis([0, 2e3, 0, 2e3, 0, 2e3])
        i = 0
        t1 = time.time()
        t_total = 0
        dt = 0
        exp_len = struct.calcsize(ENDIAN+FORMAT)
        ser.flushInput()
        t = np.array(())

        while ser.isOpen and t_total<10:
            ser.write('1')
            ser.flush()
            time.sleep(0.005)
            buff = SC.Receiving()
            if len(buff) == 0:
                continue
            if np.ndim(buff)>1:
                buff = buff[-1,:]

            in_data , pkt = SC.ParsePacket(buff)
            hdr = in_data[0]
            rdata = np.array(in_data[1])
            chksum = in_data[2]
            ftr = in_data[3]
            chksum_in = SC.GetChecksum(pkt)
            gdata = f_map(rdata, 0.0, 1024.0, -g_scale, g_scale)
            adata = gdata*gravity

            if i is 0:
                acc_arr = adata
                rdata_arr = rdata
            else:
                acc_arr = np.row_stack((acc_arr , adata))
                rdata_arr = np.row_stack((rdata_arr, rdata))

            t2 = time.time()
            dt = t2 - t1
            t1 = t2
            t_total = t_total + dt
            t = np.append(t,t_total)
            if i is 0:
                dt_arr = dt
            else:
                dt_arr = np.append(dt_arr,dt)
            i += 1

        mu = np.mean(acc_arr,0)
        plt.ion()
        offset[adir,:] = real[adir,:] / mu
        g_offset = offset[adir,adir]
        bias[adir,:] = np.mean(acc_arr,0)*g_offset

        ax[0].plot(t,acc_arr[:,0], 'b.')
        ax[1].plot(t,acc_arr[:,1], 'r.')
        ax[2].plot(t,acc_arr[:,2], 'k.')

        plt.figure(4)
        plt.subplot(211)
        plt.plot(acc_arr*g_offset)
        plt.subplot(212)
        plt.plot(acc_arr*g_offset - bias[adir,:])

        time.sleep(5)

    np.savetxt('bias.csv', bias, delimiter = ',', fmt = '%1.4e')
    np.savetxt('offset.csv', offset, delimiter = ',', fmt = '%1.4e')
    # with open('bias.csv', 'w') as bfile:
    #     writer = csv.writer(bfile, delimiter = ',')
    #     writer.writerow(bias)
    # with open('offset.csv', 'w') as ofile:
    #     writer = csv.writer(ofile, delimiter = ',')
    #     writer.writerow(offset)
    plt.show()
    ser.close()
