import numpy as np
import math as m
import serial
from SerialCommunication import SerialCommunication
import time
import csv
import struct
from scipy.signal import butter, lfilter, freqs
# from scipy.fftpack import fft, ifft

from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from IPython.core.debugger import set_trace as trc

ENDIAN = '@'
FORMAT = '8H'

def butter_lowpass(cutoff, fs, order=5):
    nyq = 0.5 * fs
    normal_cutoff = cutoff / nyq
    b, a = butter(order, normal_cutoff, btype='low', analog=False)
    return b, a

def butter_lowpass_filter(data, cutoff, fs, order=5):
    b, a = butter_lowpass(cutoff, fs, order=order)
    y = lfilter(b, a, data)
    return y

def moving_average(data, n):
    buffer = [np.nan]*n
    for i in range(n, len(signal)):
        buffer.append(signal[i-n:i].mean())
    return buffer

def f_map(x, in_min, in_max, out_min, out_max):
  map_vals = (x - in_min) * (out_max - out_min) \
          / (in_max - in_min) + out_min;
  return map_vals

if __name__ == "__main__":
    plt.close('all')
    SC = SerialCommunication()
    ser = SC.ser
    i = 0
    buff = ''

    gravity = 9.81
    g_scale = 3.0
    fig1, ax1 = plt.subplots(3,2, num = 1)
    fig2 = plt.figure(2)
    ax2 = fig2.add_subplot(111, projection='3d')
    # ax.axis([0, 2e3, 0, 2e3, 0, 2e3])
    t1 = time.time()
    t_total = 0
    dt = []
    t = np.array(())
    tmp = np.array(())

    order = 1
    fs = 565
    cutoff = 0.5
    ma_n = 16

    b,a = butter_lowpass(cutoff, fs, order)

    bias = np.loadtxt('bias.csv', delimiter = ',')
    bias = bias[-1,:]

    offset = np.loadtxt('offset.csv', delimiter = ',')
    offset = offset.diagonal()

    exp_len = struct.calcsize(ENDIAN+FORMAT)
    ser.flushInput()
    displacement = np.zeros((1,3))
    velocity = np.zeros((1,3))
    print('Calibration start: ')
    while ser.isOpen and t_total<10:
        ser.write('1')
        # ser.flush()
        time.sleep(0.005)
        buff = SC.Receiving()
        # nrow,ncol = buff.shape
        # for vec in buff:
        if len(buff) == 0:
            continue
        elif np.ndim(buff) > 1:
            buff = buff[-1,:]
        # if type(buff) is tuple:
        in_data , pkt = SC.ParsePacket(buff)
        hdr = in_data[0]
        rdata = np.array(in_data[1])
        chksum = in_data[2]
        ftr = in_data[3]
        chksum_in = SC.GetChecksum(pkt)
        gdata = f_map(rdata, 0.0, 1024.0, -g_scale, g_scale)
        adata = gdata*offset*gravity - bias
        if t_total == 0:
            tmp = adata
        else:
            tmp = np.row_stack((tmp,adata))
        t_total += time.time() - t1
        dt.append(time.time() - t1)
        t1 = time.time()

    t1 = time.time()
    t_total = 0
    mu = np.mean(tmp,0)
    sd = np.std(tmp,0)
    thold = 3*sd
    # bias += np.mean(tmp,0)

    dt = 0
    print('calibration done')
    while ser.isOpen and t_total<10:
        ser.write('1')
        # ser.flush()
        # time.sleep(0.005)
        buff = SC.Receiving()
        # nrow,ncol = buff.shape
        # for vec in buff:
        if len(buff) == 0:
            print('skip')
            continue
        elif np.ndim(buff) > 1:
            buff = buff[-1,:]
        in_data , pkt = SC.ParsePacket(buff)
        hdr = in_data[0]
        rdata = np.array(in_data[1])
        chksum = in_data[2]
        ftr = in_data[3]
        chksum_in = SC.GetChecksum(pkt)
        gdata = f_map(rdata, 0.0, 1024.0, -g_scale, g_scale)
        adata = gdata*offset*gravity - bias - mu

        # adata_lp = butter_lowpass_filter(adata, cutoff, fs, order)
        # accel = np.round(adata_lp,3)

        # Begin moving average if we have the minimum number of samples
        # required
        accel = adata

        # if i <= ma_n and i > 0:
        #     acc_ma = acc_arr
        # elif i > ma_n:
        #     accel = sum(acc_arr[i-ma_n:i,:])/ma_n
        if i > ma_n:
            accel_ma = sum(acc_arr[i-ma_n:i,:])/ma_n
            mask = abs(accel_ma) < abs(thold)
            accel = accel_ma
        else:
            mask = abs(accel) < abs(thold)

        if sum(mask) > 0:
            accel[mask] = 0

        # mask = adata_lp < thold
        # if sum(mask) > 0:
        #     adata_lp[mask] = 0

        if i == 0:
            velocity = np.array(accel*dt)
            displacement = np.array(displacement + velocity*dt)
        else:
            if np.ndim(velocity) == 1:
                vel0 = velocity
                vel1 = (dt/2.0)*(accel + acc_arr)
            else:
                vel0 = velocity[-1,:]
                vel1 = (dt/2.0)*(accel + acc_arr[-1,:])

            disp1 = displacement[-1,:] + (dt/2.0)*(vel1 + vel0)

            velocity = np.row_stack((velocity, vel1))
            displacement = np.row_stack((displacement, disp1))
        # else:
        #     for j,vec in enumerate(buff):
        #         in_data , pkt = SC.ParsePacket(vec)
        #         hdr = in_data[0]
        #         rdata = np.array(in_data[1])
        #         chksum = in_data[2]
        #         ftr = in_data[3]
        #         chksum_in = SC.GetChecksum(pkt)
        #         gdata = f_map(rdata, 0.0, 1024.0, -g_scale, g_scale)
        #         a = gdata*gravity - bias
        #         if j == 0:
        #             adata = a
        #         else:
        #             adata = np.row_stack((adata,a))

        #         displacement = np.round(adata,2)*(pow(dt,2)/2)
        #         # ax[0].plot(t_total,adata[0], 'b.')
        #         # ax[1].plot(t_total,adata[1], 'r.')
        #         # ax[2].plot(t_total,adata[2], 'k.')
        #         ax[0].plot(t_total,a[0], 'b.')
        #         ax[1].plot(t_total,a[1], 'r.')
        #         ax[2].plot(t_total,a[2], 'k.')

        # Let's initialise our buffer arrays
        if i is 0:
            acc_arr = adata
            rdata_arr = rdata
        else:
            acc_arr = np.row_stack((acc_arr , adata))
            rdata_arr = np.row_stack((rdata_arr, rdata))
            if i > ma_n:
                acc_ma = np.row_stack((acc_ma, accel_ma))
            else:
                acc_ma = acc_arr

        t2 = time.time()
        dt = t2 - t1
        t1 = t2
        t_total = t_total + dt
        t = np.append(t,t_total)
        if i is 0:
            dt_arr = dt
        else:
            dt_arr = np.append(dt_arr,dt)
        i += 1

    ax1[0,0].plot(t,acc_arr[:,0], 'b.')
    ax1[0,1].plot(t,acc_ma[:,0], 'b-')
    ax1[1,0].plot(t,acc_arr[:,1], 'r.')
    ax1[1,1].plot(t,acc_ma[:,1], 'r-')
    ax1[2,0].plot(t,acc_arr[:,2], 'k.')
    ax1[2,1].plot(t,acc_ma[:,2], 'k-')

    plt.figure(2)
    plt.subplot(311)
    plt.plot(t,displacement[:,0], 'b.')
    plt.subplot(312)
    plt.plot(t,displacement[:,1], 'r.')
    plt.subplot(313)
    plt.plot(t,displacement[:,2], 'k.')

    plt.figure(3)
    # plt.subplot(211)
    plt.plot(acc_arr)
    # plt.subplot(212)
    # plt.plot(acc_arr)

    fig2 = plt.figure(4)
    ax2 = fig2.add_subplot(111, projection='3d')
    for d in displacement:
        ax2.scatter(d[0], d[1], d[2], c='k', marker='.')
    ax2.set_xlabel('x')
    ax2.set_ylabel('y')
    ax2.set_zlabel('z')

    np.savetxt('rdata.csv',rdata_arr,delimiter=',')
    # with open('rdata.csv', 'a+') as rfile: 
    #     writer = csv.writer(rfile, delimiter = ',')
    #     writer.writerow(rdata_arr)
    ser.close()
    plt.ion()
    plt.show()
