import numpy as np
import math as m
import serial
import time
import csv
import struct
from scipy.signal import butter, lfilter, freqs

from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from IPython.core.debugger import set_trace as trc

ENDIAN = '<'
FORMAT = 'h2b3hbh'

def f_map(x, in_min, in_max, out_min, out_max):
  map_vals = (x - in_min) * (out_max - out_min) \
          / (in_max - in_min) + out_min;
  return map_vals

if __name__ == "__main__":
    i = 0
    buff = ''

    gravity = 9.81
    g_scale = 3.0
    fig = plt.figure(1)
    ax = fig.add_subplot(111, projection='3d')
    # ax.set_xlim([-3, 3])
    # ax.set_ylim([-3, 3])
    # ax.set_zlim([0, 6])
    fig1, ax1 = plt.subplots(3,1, num = 2)
    dt = 0.1

    with open('bias.csv') as cfile:
        fread = csv.reader(cfile, delimiter = ',')
        [bias_str for bias_str in fread]
    bias = [float(s) for s in bias_str]

    with open('offset.csv') as ofile:
        fread = csv.reader(ofile, delimiter = ',')
        [offset_str for offset_str in fread]

    offset = [float(s) for s in offset_str]
    offset = offset[-1]

    rdata_arr = np.loadtxt('rdata.csv', delimiter = ',')

    gdata = f_map(rdata_arr, 0.0, 1024.0, -g_scale, g_scale)
    adata = gdata*gravity - bias
    # adata[:,2] = adata[:,2] - 9.81
    bias1 = np.mean(adata[0:11,:],0)
    adata = adata - bias1
    displacement = np.zeros_like(adata)

    t = range(0,len(adata))
    for i,a in enumerate(adata):
        if i == 0:
            displacement[i,:] = np.zeros((1,3))
        else:
            displacement[i,:] = displacement[i-1,:] + \
                np.round(a,2)*(pow(dt,2)/2)

    for i in range(0,3):
        ax1[i].plot(t,displacement[:,i])

    for d in displacement:
        cm = d*100
        ax.scatter(cm[0], cm[1], cm[2],c='k')
        plt.pause(0.001)

    plt.ion()
    plt.show()
